TrackCooldowns = LibStub("AceAddon-3.0"):NewAddon("TrackCooldowns", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("TrackCooldowns")

local UnitName = _G.UnitName

function TrackCooldowns:OnInitialize()
	self:UnregisterAllEvents()
	self:SetEnabledState(false)
	self.prefix = "TC"
	self:RegisterEvent("PLAYER_LOGIN")
end

function TrackCooldowns:OnEnable()
	self:RegisterComm(self.prefix)
	self:RegisterComm("oRA")
	self:RegisterComm("CTRA")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
	local name, module
	for name, module in self:IterateModules() do
		if not module:IsEnabled() then
			self:EnableModule(name)
		end
	end
	
	self:Print(L["is loaded."])
	self:Print(L["type %s to get help."]:format("|cff66ccff/tc|r"))
end

function TrackCooldowns:OnDisable()
end

--------------[[		Events		]]--------------

function TrackCooldowns:PLAYER_LOGIN()
	if UnitInRaid("player") or UnitInBattleground("player") then
		self:RegisterEvent("RAID_ROSTER_UPDATE")
		self:RAID_ROSTER_UPDATE()
	elseif UnitInParty("player") then
		self:RegisterEvent("PARTY_MEMBERS_CHANGED")
		self:PARTY_MEMBERS_CHANGED()
	end	
end

do
	local enabled = false

	function TrackCooldowns:RAID_ROSTER_UPDATE()
		if UnitInRaid("player") and self:GetModule("Display").db.profile.inRaids then
			self:Enable()
			enabled = true
		elseif not enabled then
			self:Disable()
		end
	end

	function TrackCooldowns:PARTY_MEMBERS_CHANGED()
		if (UnitInParty("player") and self:GetModule("Display").db.profile.inParties) or (UnitInBattleground("player") and self:GetModule("Display").db.profile.inBattlegrounds) then
			self:Enable()
			enabled = true
		elseif not enabled then
			self:Disable()
		end
	end

end

function TrackCooldowns:COMBAT_LOG_EVENT_UNFILTERED(_, timestamp, event, hideCaster, srcGUID, srcName, srcFlags, srcRaidFlags, destGUID, destName, destFlags, destRaidFlags, spellID, spellName, spellSchool, extraSpellID, extraSpellName, extraSpellSchool, auraType)

	if event ~= "SPELL_CAST_SUCCESS" and event ~= "SPELL_RESURRECT" then return end
	if not srcName then return end

	local _, c = UnitClass(srcName)
	if self.cooldowns[c] then
		local spells = self.cooldowns[c]
		if spells[spellName] then
			self:GetModule("Display"):StartCooldown(srcName, spellID, spells[spellName].cd)
			return
		end
	end
	
	return
end

--------------[[		Comm Methods		]]--------------

function TrackCooldowns:OnCommReceived(prefix, msg, distro, sender)
	if self:GetModule("Display").db.profile.hideSelf and sender == playerName then return end
	if self:GetModule("Display").db.profile.showOwn and sender ~= playerName then return end
	
	local spellId, cooldown = 0, 0
	
	if prefix == "oRA" or prefix == "CTRA" then
		spellId, cooldown = select(3, msg:find("CD (%d) (%d+)"))
		spellId  = tonumber(spellId)
		cooldown = tonumber(cooldown)
		
		if spellId == 0 or cooldown == 0 then return end
		
		local _, c = UnitClass(sender)
		if self.cooldowns[c] then
			for k, v in pairs(self.cooldowns[c]) do
				if v.ora and v.ora == spellId then
					spellId = v.id
					self:GetModule("Display"):StartCooldown(sender, spellId, v.cd)
					return
				end
			end
		end

	else
		spellId, cooldown = select(3, msg:find("(%d+) (%d+)"))
		spellId  = tonumber(spellId)
		cooldown = tonumber(cooldown)
		
		if cooldown == 0 then
			self:GetModule("Display"):StopCooldown(sender, spellId)
		else
			self:GetModule("Display"):StartCooldown(sender, spellId, cooldown)
		end

	end
end